from colr import color
from time import ctime
__version__ = "1.2.0"


class Manager:
    def __init__(self, priority:int=-1, maxLogCount:int=1000):
        """
        Initialise the Manager and then use the public functions to display coloured logs
        :param priority: Integer value to set minimum threshold of log verbosity
        :param maxLogCount: Maximum number of logs to store in memory, Values available as list in `Manager().logs`
        """
        self._priority = priority
        self.logs = []
        self._maxLogCount = maxLogCount

        self._fatal = [5, (255, 0, 0), (90, 0, 0)]
        self._failed = [4, (255, 182, 0), (127, 89, 0)]
        self._success = [3, (57, 202, 0), (23, 81, 0)]
        self._info = [2, (0, 140, 255), (0, 52, 95)]
        self._skip = [1, (0, 255, 195), (0, 100, 77)]


    def __log(self, string: str, back: tuple[int,int,int], fore: tuple[int,int,int]):
        """
        Private function to finally print the coloured text onto terminal and remove older logs if exceeds max count.
        :param string: The string to print
        :param back: RGB tuple specifying the background colour
        :param fore: RGB tuple specifying the foreground colour
        :return:
        """
        self.logs.append(string)
        if len(self.logs) > self._maxLogCount:
            self.logs.pop(0)
        print(color(string, back=back, fore=fore))


    @staticmethod
    def __formString(category: str, *args:*list[str]):
        """
        Decorate the log string and all other arguments passed into a single string
        :param category: The string that gets decorated
        :param args: Other strings to join to fina log
        :return:
        """
        return f"[{ctime()}] [{category}] {' '.join(args)}"


    def skip(self, category: str, *args):
        """
        Log with verbose value 1
        :param category:
        :param category: The string that gets decorated
        :param args: Other strings to pass
        """
        string = self.__formString(category, *args)
        if self._skip[0] >= self._priority:
            self.__log(
                string,
                back=self._skip[1],
                fore=self._skip[2],
            )
        return string


    def info(self, category: str, *args):
        """
        Log with verbose value 2
        :param category:
        :param category: The string that gets decorated
        :param args: Other strings to pass
        """
        string = self.__formString(category, *args)
        if self._info[0] >= self._priority:
            self.__log(
                string,
                back=self._info[1],
                fore=self._info[2],
            )
        return string


    def success(self, category: str, *args):
        """
        Log with verbose value 3
        :param category:
        :param category: The string that gets decorated
        :param args: Other strings to pass
        """
        string = self.__formString(category, *args)
        if self._success[0] >= self._priority:
            self.__log(
                string,
                back=self._success[1],
                fore=self._success[2],
            )
        return string


    def failed(self, category: str, *args):
        """
        Log with verbose value 4
        :param category:
        :param category: The string that gets decorated
        :param args: Other strings to pass
        """
        string = self.__formString(category, *args)
        if self._failed[0] >= self._priority:
            self.__log(
                string,
                back=self._failed[1],
                fore=self._failed[2],
            )
        return string


    def fatal(self, category: str, *args):
        """
        Log with verbose value 5
        :param category:
        :param category: The string that gets decorated
        :param args: Other strings to pass
        """
        string = self.__formString(category, *args)
        if self._fatal[0] >= self._priority:
            self.__log(
                string,
                back=self._fatal[1],
                fore=self._fatal[2],
            )
        return string

